# --- animate.tcl --- Demonstration for the Animate extension under Windows

# Copyright(C) I.B.Findleton, 1999. All Rights Reserved

# This script will accept a file name filter and then loop through all of the images
# it finds, displaying them in the picture window. The default filter is pointing at
# the standard c:/windows directory, which usually has loads of images...

package require Animate 1.0

catch { destroy .f .actions }

update idletasks

set f [frame .f -height 500 -width 500 -relief raised -bd 2]

set f1 [frame .actions -relief groove -bd 2]
button $f1.next -width 8 -relief flat -text "Next" -command "Next"
button $f1.previous -width 8 -relief flat -text "Previous" -command "Previous"
button $f1.forward -width -8 -relief flat -text "Forward" -command "Forward"
button $f1.backward -width 8 -relief flat -text "Backward" -command "Backward"
button $f1.quit -width 8 -relief flat -text "Quit" -command "exit"
button $f1.filter -width 8 -relief flat -text "Filter" -command "Filter"
button $f1.config -width 8 -relief flat -text "Options" -command "Config"

pack $f1.next $f1.previous $f1.forward $f1.backward $f1.filter $f1.config  -side left
pack $f1.quit -side right

pack $f1 -side top -fill x -expand yes
pack $f -side top -fill both -expand yes

# Call update to ensure that the windows are mapped.

update idletasks

global filter

set filter "c:/windows/*.bmp"

set Animation(Direction) "forward"
set Animation(List) [glob $filter]
set Animation(Current) 0

# The variable option sets the name of the animation sync variable to Animation(Painted,$w)

set Animation(Window) [picture $f.image -variable "Animation,Painted"]

set Animation(Running) 0

# The following are the default values

set Animation(Fit) false
set Animation(Background) erase
set Animation(Center) false
set Animation(ScrollBars) true

proc CheckOptions {} {

	global Animation

	if { $Animation(Center) == "true" } {
		set Animation(ScrollBars) "false"
		}
	if { $Animation(ScrollBars) == "true" } {
		set Animation(Center) "false"
		}
	}

# Configure the options for the display window

proc Config {} {

	global Animation

	catch { destroy .prompt }

	set f [toplevel .prompt]
	set f1 [frame $f.options -bd 2 -relief ridge]
	checkbutton $f1.background -variable Animation(Background) -onvalue "erase" -offvalue "noerase" \
					-text "Erase the background when updating the image window" -anchor w
	checkbutton $f1.fit -variable Animation(Fit) -onvalue "true" -offvalue "false" \
					-text "Automatically stretch the image to the window" -anchor w
	checkbutton $f1.center -variable Animation(Center) -onvalue "true" -offvalue "false" \
					-text "Center the image in the window" -anchor w -command CheckOptions
	checkbutton $f1.scroll -variable Animation(ScrollBars) -onvalue "true" -offvalue "false" \
					-text "Enable the display of scroll bars in window" -anchor w -command CheckOptions

	pack $f1.background $f1.fit $f1.center $f1.scroll -side top -fill x -anchor w -expand yes

	set f2 [frame $f.actions -bd 2 -relief ridge]
	button $f2.ok -width 8 -text "Accept" -command { destroy .prompt }

	pack $f2.ok -side left

	pack $f1 $f2 -side top -fill x -expand yes

	wm title .prompt "Animation Configuration"

	tkwait window .prompt

	$Animation(Window) config -background $Animation(Background) -fit $Animation(Fit) \
					-center $Animation(Center) -scrollbars $Animation(ScrollBars)

	}

# Set the state of the buttons used to control the animation

proc EnableButtons {} {

	global Animation

	set f1 .actions

	if { $Animation(Running) } {
		$f1.next config -state disabled
		$f1.previous config -state disabled
		$f1.quit config -state disabled
		$f1.filter config -state disabled
		$f1.config config -state disabled
		if { $Animation(Direction) == "forward" } {
			$f1.forward config -text Stop -state normal
			$f1.backward config -state disabled
		} else {
			$f1.forward config -state disabled
			$f1.backward config -state normal -text Stop
			}
	} else {
		$f1.next config -state normal
		$f1.previous config -state normal
		$f1.quit config -state normal
		$f1.filter config -state normal
		$f1.forward config -text Forward -state normal
		$f1.backward config -text Backward -state normal
      $f1.config config -state normal
		}
	}

# DIsplay the next image in the list

proc Next {} {
	global Animation

	set len [llength $Animation(List)]

	if { [incr Animation(Current)] >= $len} {
		set Animation(Current) 0
		}

	set name [lindex $Animation(List) $Animation(Current)]
	wm title . "File $name"

	catch { $Animation(Window) config -file $name }

	tkwait variable Animation(Painted,$Animation(Window))
	}

# Display the previous image in the list

proc Previous {} {

	global Animation

	set len [llength $Animation(List)]

	if { [set Animation(Current) [expr $Animation(Current) - 1]] < 0 } {
		set Animation(Current) [expr $len - 1]
		}

	set name [lindex $Animation(List) $Animation(Current)]
	wm title . "File $name"

	catch { $Animation(Window) config -file $name }
	tkwait variable Animation(Painted,$Animation(Window))
	}

# Toggle the state of the animation loop

proc ToggleRunState { direction } {

	global Animation

	if { $Animation(Running) } {
		set Animation(Running) 0
	} else {
		set Animation(Running) 1
		set Animation(Direction) $direction
		}

	return $Animation(Running)
	}

# Run the animation forward

proc Forward {} {

	global Animation

	ToggleRunState "forward"

	EnableButtons

	while { $Animation(Running) } {
		Next
		}
	}

# Run the animation in reverse

proc Backward {} {

	global Animation

	ToggleRunState "backward"

	EnableButtons

	while { $Animation(Running) } {
		Previous
		}
	}

# Get a new filter string

proc Filter {} {

	global Animation filter

	catch { destroy .prompt }

	set w [toplevel .prompt]
	wm title .prompt "Animation File Filter"
	entry $w.entry -width 20 -font "Fixedsys 10" -textvariable filter
	button $w.ok -width 6 -text Ok -command { destroy .prompt }

	pack $w.entry $w.ok -side left

	bind $w.entry <Return> { destroy .prompt }

	tkwait window .prompt

	set Animation(List) [glob $filter]
	set Animation(Current) 0
	}

